#!/usr/bin/perl

#/***************************************************************************
#**
#** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
#** All rights reserved.
#** Contact: Karoliina T. Salminen <karoliina.t.salminen@nokia.com>
#**
#** This file is part of duicontrolpanel-devmodeapplet.
#**
#**
#** This library is free software; you can redistribute it and/or
#** modify it under the terms of the GNU Lesser General Public
#** License version 2.1 as published by the Free Software Foundation
#** and appearing in the file LICENSE.LGPL included in the packaging
#** of this file.
#**
#****************************************************************************/

use strict;
use warnings;

my ($xpidfile, $xmsg, $xreboot) = ('', '', 0);

#open STDERR, '>', '/dev/tty'; ## not needed anymore
#print STDERR "$0 ARGV: @ARGV\n";

sub dbus_sig_devmode($$)
{
    system qw(dbus-send --session --type=signal --dest=com.nokia.devmodeapplet
 	/ com.nokia.devmodeapplet.devmode_signal),
	"string:$_[0]", "string:$_[1]";
}

END {
    unlink $xpidfile if $xpidfile;
    dbus_sig_devmode 'abort', $xmsg if $xmsg;
    system '/sbin/reboot' if $xreboot;
}    

sub exitzero() 
{
    $xmsg = '';
    exit 0;
}

sub xdie(@)
{
    $xmsg = join '', @_;
    die $xmsg, "\n";
}

sub xopenI($)
{
    xdie "Cannot open '$_[0]' for reading: $!" unless open I, '<', $_[0];
}

sub xopenO($)
{
    xdie "Cannot open '$_[0]' for writing: $!" unless open O, '>', $_[0];
}

my $cpid = 0;
sub xpIopen(@)
{
    sub _xfork()
    {
	my $pid = fork();
	die "fork() failed: $!\n" unless defined $pid;
	return $pid;
    }

    pipe I, WRITE or die "pipe() failed: $!\n";
    if ($cpid = _xfork) {
	# parent;
	close WRITE;
	return;
    }
    # child
    close I;
    open STDOUT, ">&WRITE" or die "dup2() failed: $!\n";
    exec @_;
    die "execve() failed: $!\n";
}

sub xpIclose()
{
    kill 9, $cpid;
    close I;
    return wait;
}

my $repodir = '/home/user/MyDocs/harmattan-developer-repo';
sub install_package($)
{
    if (! -d $repodir and -f "$repodir.tar.gz")
    {
	system 'rm', '-rf', "$repodir.x";
	mkdir "$repodir.x";
	dbus_sig_devmode 'message', 'extracting repodir';
	system 'tar', '-C', "$repodir.x", '-zxf', "$repodir.tar.gz";
	unlink '/etc/apt/sources.list.d/devrepo-local.list';
	rename "$repodir.x", $repodir;
    }
    if  (-d $repodir and ! -f '/etc/apt/sources.list.d/devrepo-local.list') {
	xopenO '/etc/apt/sources.list.d/devrepo-local.list';
	print O "deb file://$repodir binary/\n";
	close O;
	system qw(apt-get update); ## may work, or then not
	#system qw( pkgmgr refresh-cache -a); ## alternative
    }
    dbus_sig_devmode 'message', "installing $_[0]";

    # XXX as hacky as it can be ... XXX
    xpIopen 'dbus-monitor',
    '--session', 'interface=org.freedesktop.DBus,member=NameOwnerChanged';

    system qw(dbus-send --session --type=method_call --print-reply
	--dest=com.nokia.package_manager_install_ui 
	/com/nokia/package_manager_install_ui 
	com.nokia.package_manager_install_ui.install_package), 
	"string:$_[0]", 'string:', 'string:', 'string:showoperationspage';

    # XXX ... XXX
    while (<I>) {
	if (/string\s+"com.nokia.package_manager_install_ui"/) {
	    #$_ = <I>; next unless /string\s+""/;
	    $_ = <I>; last if /string\s+":\d.*"/;
	}
    }
    xpIclose;
    return;

    system qw/pkgmgr install -p/, $_[0];
    return;

    system qw(pkill pkgmgrd);
    sleep 1;
    system qw(apt-get install), $_[0];

    return;

    system qw(dbus-send --session --type=method_call 
	--dest=com.nokia.package_manager_ui /com/nokia/package_manager_ui
	com.nokia.package_manager_ui.show_installed_page), "string:$_[0]";
}

my %installed_packages;
sub read_installed_packages()
{
    xopenI '/var/lib/dpkg/status';
    my $pkg;
    while (<I>) {
	if (defined $pkg) {
	    if (/^Status:.*\sinstalled\s/) { # trailing \n belongs in \s.
		$installed_packages{$pkg} = 1;
		undef $pkg;
		next;
	    }
	    undef $pkg, next if /^\s*$/;
	}
	$pkg = $1 if /^Package:\s+(\S+)/;
    }
    close I;
}

my $pidfile = '/var/lib/developer-mode/devmode.pid';
sub mkpidfile ($)
{
    mkdir '/var/lib/developer-mode' unless -d '/var/lib/developer-mode';
    symlink $$, $pidfile;
    $xpidfile = $pidfile;
    $xmsg = $_[0];
}

die "$0 operation [args]\n" unless @ARGV > 0;

if ($ARGV[0] eq 'chkpid')
{
    print("0\n"), exit 0 unless -l $pidfile;
    my $pid = readlink $pidfile;
    unlink $pidfile, print("0\n"), exit 0 unless -d "/proc/$pid";
    xopenI "/proc/$pid/cmdline";
    read I, $_, 4096;
    close I;
    unlink $pidfile, print("0\n"), exit 0 unless (index $_, $0) >= 0;
    print "$pid\n";
    exit 0;
}
elsif ($ARGV[0] eq 'toolpkgs') 
{
    read_installed_packages;
    xopenI '/usr/lib/developer-mode/developer-mode.pkglist';
    while (<I>) {
	/(\S+)\s+(\S+)\s+(.*?)\s*$/;
	my $f = (defined $installed_packages{$1})? 1: 0;
	print "$1 $f $2 $3\n";
    }
    close I;
    exit 0;
}
elsif ($ARGV[0] eq 'install') 
{
    mkpidfile "installing $ARGV[1] failed";
    xdie "install <pkg-name>\n" unless defined $ARGV[1];
    install_package $ARGV[1];
    read_installed_packages;
    xdie "failed to install $ARGV[1]" 
	unless defined $installed_packages{$ARGV[1]};
    dbus_sig_devmode 'installed', $ARGV[1];
    exitzero;
}
elsif ($ARGV[0] eq 'needreboot') 
{
    read_installed_packages;
    print(((defined $installed_packages{'aegis-dev-mode'})? 'no':'yes'), "\n");
    exit 0;
}
elsif ($ARGV[0] eq 'enable') 
{
    mkpidfile 'enable failed';
    read_installed_packages;
    my $havedevmode = defined $installed_packages{'aegis-dev-mode'};
    install_package 'developer-mode' unless defined $installed_packages{'developer-mode'};

    read_installed_packages;
    $xreboot = 1 if (!$havedevmode && defined $installed_packages{'aegis-dev-mode'});

    xdie 'developer mode missing' unless defined $installed_packages{'developer-mode'};
    
    system qw(/bin/sh /usr/lib/developer-mode/bin/developer-mode.sh enable);
    system qw(gconftool -s /Meego/System/DeveloperMode --type=string true);
    dbus_sig_devmode 'master', 'enabled';
    exitzero;
}
elsif ($ARGV[0] eq 'disable') 
{
    mkpidfile 'disable failed';
    system qw(/bin/sh /usr/lib/developer-mode/bin/developer-mode.sh disable);
    system qw(gconftool -s /Meego/System/DeveloperMode --type=string false);
    dbus_sig_devmode 'master', 'disabled';
    exitzero;
}

die "'$ARGV[0]': unknown operation\n";
